--
-- ItemManager - Lotro Plugin to help manage subsets of items in backpack in windows
--
-- Copyright (C) 2012   Random Augury
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	
--	You may contact me at: randomaugury@gmail.com

import "Turbine.UI";
import "Turbine.Gameplay";

import "Augury.ItemManager.Class";
import "Augury.ItemManager.Constants";

import "Augury.ItemManager.ItemFilter";
import "Augury.ItemManager.IMDisplay";

import "Augury.ItemManager.OptionsMenu";


IMSet = class(Turbine.UI.ListBox);

function IMSet:Constructor(_setKey)
	Turbine.UI.ListBox.Constructor(self); -- Call base constructor
	
	
	self.setKey = _setKey; -- the setKey is used as part of the filename for loading from data files
	self.itemFilter = ItemFilter(_setKey);
	self.imDisplay = IMDisplay(_setKey);
	
	self:CreateScrollBar();
	
	self:SetInitialLayout();
	self:ReloadItems();

	self:AttachListeners();
end;

function IMSet:CreateScrollBar()
	self.scrollBar = Turbine.UI.Lotro.ScrollBar();
	self.scrollBar:SetParent(self.imDisplay);
	self.scrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
	self:SetVerticalScrollBar(self.scrollBar);
end;

function IMSet:AttachListeners()
	-- These are made up as lambda functioins, to make sure that the correct
	-- instance of each object gets the event.
	AddListener(self.imDisplay, "DragDrop", function(sender, args)
		self.itemFilter.DragDropHandler(self.itemFilter, sender, args);
	end);
	AddListener(self.imDisplay, "SizeChanged", function(sender, args)
		self:Resize();
	end);
	AddListener(self.imDisplay, "Closed", function(sender, args)
		self.imDisplay:OnClosed(sender, args);
	end);	
	AddListener(self.itemFilter, "FilterChanged", function(sender, args)
		self:ReloadItems();		
	end);
	
	-- Clicking on the window will pop up the menu to allow you to change things
	AddListener(self, "MouseClick", function(sender, args)
		OptionsMenu:AttachSet(self);
		OptionsMenu:ShowMenu();
	end);	
	AddListener(self.imDisplay, "MouseClick", function(sender, args)
		OptionsMenu:AttachSet(self);
		OptionsMenu:ShowMenu();
	end);	
	
	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	AddListener(backpack, "ItemAdded", function(sender, args)
		self:ItemAddedHandler(sender, args);
	end);

	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	AddListener(backpack, "ItemMoved", function(sender, args)
		self:ItemMovedHandler(sender, args);
	end);
	AddListener(backpack, "ItemRemoved", function(sender, args)
		local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
		local theItem = backpack:GetItem(args.Index);
	
		self:ItemRemovedHandler(sender, args);
	end);
	
end;


-- Clear the items in the IMSet, and reloads it with all of the items in the player's
-- backpack that match the associated filter.
function IMSet:ReloadItems()
	Debug:WriteLn(5, "Entering IMSet:ReloadItems");
	self:ClearItems();
	
	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	for i = 1, backpack:GetSize(), 1 do
		
		local theItem = backpack:GetItem(i);
		self:AddItemIfMatching(theItem);
	end	
end;

function IMSet:AddItemIfMatching(theItem)
	if (theItem ~= nil and self.itemFilter:IsItemAMatch(theItem))
	then
		self:AddItem(Turbine.UI.Lotro.ItemControl(theItem));
		Debug:WriteLn(3, "AddItemIfMatching(): item = " .. tostring(theItem) .. ", " .. theItem:GetName());
	end
end;

function IMSet:SetName(newName)
	self.imDisplay:SetName(newName);
end;

function IMSet:GetName()
	return self.imDisplay:GetName();
end;

-- Resizes the IMSet based on the dimensions of the bounding parent IMDisplay (a window specialization)
function IMSet:Resize()
	Debug:WriteLn(5, "Entering IMSet:Resize");
	local width, height = self.imDisplay:GetSize();
	
	local listWidth = width - Constants.xPadding;
	local listHeight = height - Constants.yPadding;
	local itemsPerRow = listWidth / Constants.itemWidth;

	self:SetSize(listWidth, listHeight);
	self:SetMaxItemsPerLine(itemsPerRow);
	
	self.scrollBar:SetPosition(width - 20, 40);
	self.scrollBar:SetSize(10, listHeight - 15);
	
end;

-- Set the initital position of the ImSet. Currently this does not change as its logically
-- bound to the top left corner.
function IMSet:SetInitialLayout()
	Debug:WriteLn(5, "Entering IMSet:SetInitialLayout");
	self:SetParent(self.imDisplay);
	self:Resize();
	self:SetPosition(15, 35);
	self:SetOrientation(Turbine.UI.Orientation.Horizontal);
	self:SetVisible(true);
end;

function IMSet:ItemAddedHandler(sender, args)
	self:AddItemIfMatching(args.Item);
end;

function IMSet:ItemMovedHandler(sender, args)

	-- When an item is moved, it gets a new ID. The simplest way to handle this is to
	-- refresh the IMSet with the backpack contents.
	if (args.Item ~= nil and self.itemFilter:IsItemAMatch(args.Item))
	then
		self:ReloadItems();
	end;
end;

function IMSet:ItemRemovedHandler(sender, args)
	local backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	local theItem = backpack:GetItem(args.Index);
	
	if (theItem ~= nil and self.itemFilter:IsItemAMatch(theItem))
	then
		self:RemoveItem(theItem);
	end;
end;

-- Sadly we can't just use the RemoveItem and ContainsItem methods on the listbox,
-- as we have the item included inside an item control. The contains will match the
-- control, not the item. Therefore we are going to iterate over the whole lot and pull it out!
function IMSet:RemoveItem(theItem)
	local i = 1
	
	while (i <= self:GetItemCount())
	do
		local itemControl = self:GetItem(i);
		local currentItem = Turbine.UI.Lotro.ItemControl.GetItem(itemControl);

		if (theItem == currentItem)
		then
			self:RemoveItemAt(i);
		end;
		
		i = i + 1
	end;
end;

function IMSet:Show()
	self.imDisplay:Show();
end;
